unit ClientMainForm;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls;

type
  TForm1 = class(TForm)
    btnLateBound: TButton;
    btnEarlyBound1: TButton;
    btnEarlyBound2: TButton;
    btnEarlyBound3: TButton;
    btnEarlyBound4: TButton;
    btnEarlyBound5: TButton;
    btnEarlyBound6: TButton;
    procedure btnLateBoundClick(Sender: TObject);
    procedure btnEarlyBound1Click(Sender: TObject);
    procedure btnEarlyBound2Click(Sender: TObject);
    procedure btnEarlyBound3Click(Sender: TObject);
    procedure btnEarlyBound5Click(Sender: TObject);
    procedure btnEarlyBound6Click(Sender: TObject);
    procedure btnEarlyBound7Click(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

uses
  ComObj, Example_TLB, Client_TLB, ClientCOMClass;

{$R *.DFM}

procedure TForm1.btnLateBoundClick(Sender: TObject);
var
  //Late binding is implemented by way of a Variant
  V: Variant;
begin
  //Let Windows do all the invocation work, you supply a Prog ID
  V := CreateOleObject('Example.Server');
  //Since we are accessing the server through a Variant,
  //there is no need to do the following translation explicitly

  //ShowMessage(DateTimeToStr(V.CurrentDateAndTime));
  ShowMessage(V.CurrentDateAndTime);
end;

procedure TForm1.btnEarlyBound1Click(Sender: TObject);
var
  S: IInternalServer;
begin
  //Create object directly
  //Only works if implemented in this EXE
  S := TInternalServer.Create;
  ShowMessage(DateTimeToStr(S.CurrentDateAndTime));
end;

procedure TForm1.btnEarlyBound2Click(Sender: TObject);
var
  S: IInternalServer;
begin
  //Create object directly (sort of)
  //Only works if implemented in this EXE
  S := TInternalServer.CreateFromFactory(
         ComClassManager.GetFactoryFromClass(TInternalServer),
         nil);
  ShowMessage(DateTimeToStr(S.CurrentDateAndTime));
end;

procedure TForm1.btnEarlyBound3Click(Sender: TObject);
var
  S: IInternalServer;
begin
  //Use class factory, and knowledge of the class
  //Only works if implemented in this EXE
  S := ComClassManager.GetFactoryFromClass(TInternalServer).CreateComObject(nil) as IInternalServer;
  ShowMessage(DateTimeToStr(S.CurrentDateAndTime));
end;

procedure TForm1.btnEarlyBound5Click(Sender: TObject);
var
  S: IServer;
begin
  //Let Windows do all the factory work, you supply a ProgID
  S := CreateOleObject('Example.Server') as IServer;
  ShowMessage(DateTimeToStr(S.CurrentDateAndTime));
end;

procedure TForm1.btnEarlyBound6Click(Sender: TObject);
var
  S: IServer;
begin
  //Let Windows do all the factory work, you supply a class ID
  S := CreateComObject(Class_Server) as IServer;
  ShowMessage(DateTimeToStr(S.CurrentDateAndTime));
end;

procedure TForm1.btnEarlyBound7Click(Sender: TObject);
var
  S: IServer;
begin
  //Take advantage of the utility coclass
  //supplied in the type library import unit
  S := CoServer.Create;
  ShowMessage(DateTimeToStr(S.CurrentDateAndTime));
end;

end.
